@@ -29,7 +29,6 @@ import re
 import shutil
 import subprocess  # nosec
 import sys
-import tempfile
 import time
 import zipfile
 from collections import defaultdict
@@ -41,9 +40,8 @@ from datetime import datetime
 # It seems to be unable to import the distutils module, however the module is present and importable
 # in the Python Repl.
 from distutils.util import strtobool  # pylint: disable=E0611, E0401
-from fnmatch import fnmatch
 from importlib.abc import Loader
-from typing import Any, DefaultDict, Dict, Final, Iterator, List, Set, Tuple, Type
+from typing import Any, DefaultDict, Dict, List, Tuple, Type
 from unittest.mock import MagicMock, patch
 from uuid import uuid4
 
@@ -68,7 +66,6 @@ from ruamel.yaml import parser as YAMLParser
 from ruamel.yaml import scanner as YAMLScanner
 from schema import (
     Optional,
-    Schema,
     SchemaError,
     SchemaForbiddenKeyError,
     SchemaMissingKeyError,
@@ -76,6 +73,7 @@ from schema import (
     SchemaWrongKeyError,
 )
 
+from panther_analysis_tool.analysis_utils import filter_analysis, load_analysis_specs
 from panther_analysis_tool.backend.client import BackendError, BulkUploadParams
 from panther_analysis_tool.backend.client import Client as BackendClient
 from panther_analysis_tool.cmd import (
@@ -85,77 +83,35 @@ from panther_analysis_tool.cmd import (
     panthersdk_upload,
     standard_args,
 )
+from panther_analysis_tool.constants import (
+    CONFIG_FILE,
+    DATA_MODEL_LOCATION,
+    DATAMODEL,
+    DETECTION,
+    GLOBAL,
+    HELPERS_LOCATION,
+    LOOKUP_TABLE,
+    PACK,
+    POLICY,
+    QUERY,
+    RULE,
+    SCHEDULED_RULE,
+    SCHEMAS,
+    SET_FIELDS,
+    TMP_HELPER_MODULE_LOCATION,
+    VERSION_STRING,
+)
 from panther_analysis_tool.destination import FakeDestination
 from panther_analysis_tool.log_schemas import user_defined
 from panther_analysis_tool.schemas import (
-    DATA_MODEL_SCHEMA,
     GLOBAL_SCHEMA,
     LOOKUP_TABLE_SCHEMA,
-    PACK_SCHEMA,
     POLICY_SCHEMA,
     RULE_SCHEMA,
-    SCHEDULED_QUERY_SCHEMA,
     TYPE_SCHEMA,
 )
 from panther_analysis_tool.util import func_with_backend, get_client
-
-VERSION_STRING: Final = "panther_analysis_tool 0.17.2"
-
-CONFIG_FILE = ".panther_settings.yml"
-DATA_MODEL_LOCATION = "./data_models"
-HELPERS_LOCATION = "./global_helpers"
-LUTS_LOCATION = "./lookup_tables"
-DATA_MODEL_PATH_PATTERN = "*data_models*"
-LUTS_PATH_PATTERN = "*lookup_tables*"
-HELPERS_PATH_PATTERN = "*/global_helpers"
-PACKS_PATH_PATTERN = "*/packs"
-POLICIES_PATH_PATTERN = "*policies*"
-QUERIES_PATH_PATTERN = "*queries*"
-RULES_PATH_PATTERN = "*rules*"
-TMP_HELPER_MODULE_LOCATION = os.path.join(tempfile.gettempdir(), "panther-path", "globals")
-
-DATAMODEL = "datamodel"
-DETECTION = "detection"
-GLOBAL = "global"
-LOOKUP_TABLE = "lookup_table"
-PACK = "pack"
-POLICY = "policy"
-QUERY = "scheduled_query"
-RULE = "rule"
-SCHEDULED_RULE = "scheduled_rule"
-
-RESERVED_FUNCTIONS = (
-    "alert_context",
-    "dedup",
-    "description",
-    "destinations",
-    "reference",
-    "runbook",
-    "severity",
-    "title",
-)
-
-VALID_SEVERITIES = ["INFO", "LOW", "MEDIUM", "HIGH", "CRITICAL"]
-
-SCHEMAS: Dict[str, Schema] = {
-    DATAMODEL: DATA_MODEL_SCHEMA,
-    GLOBAL: GLOBAL_SCHEMA,
-    LOOKUP_TABLE: LOOKUP_TABLE_SCHEMA,
-    PACK: PACK_SCHEMA,
-    POLICY: POLICY_SCHEMA,
-    QUERY: SCHEDULED_QUERY_SCHEMA,
-    RULE: RULE_SCHEMA,
-    SCHEDULED_RULE: RULE_SCHEMA,
-}
-
-SET_FIELDS = [
-    "LogTypes",
-    "PackIDs",
-    "OutputIds",
-    "SummaryAttributes",
-    "Suppressions",
-    "Tags",
-]
+from panther_analysis_tool.zip_chunker import ZipArgs, ZipChunk, analysis_chunks
 
 # interpret datetime as str, the backend uses the default behavior for json.loads, which
 # interprets these as str.  This sets global config for ruamel SafeConstructor
@@ -207,87 +163,6 @@ def load_module(filename: str) -> Tuple[Any, Any]:
     return module, None
 
 
-def load_analysis_specs(
-    directories: List[str], ignore_files: List[str]
-) -> Iterator[Tuple[str, str, Any, Any]]:
-    """Loads the analysis specifications from a file.
-
-    Args:
-        directories: The relative path to Panther policies or rules.
-        ignore_files: Files that Panther Analysis Tool should not process
-
-    Yields:
-        A tuple of the relative filepath, directory name, and loaded analysis specification dict.
-    """
-    # setup a list of paths to ensure we do not import the same files
-    # multiple times, which can happen when testing from root directory without filters
-    ignored_normalized = []
-    for file in ignore_files:
-        ignored_normalized.append(os.path.normpath(file))
-
-    loaded_specs: List[Any] = []
-    for directory in directories:
-        for relative_path, _, file_list in os.walk(directory):
-            # Skip hidden folders
-            if (
-                relative_path.split("/")[-1].startswith(".")
-                and relative_path != "./"
-                and relative_path != "."
-            ):
-                continue
-            # setup yaml object
-            yaml = YAML(typ="safe")
-            # If the user runs with no path args, filter to make sure
-            # we only run folders with valid analysis files. Ensure we test
-            # files in the current directory by not skipping this iteration
-            # when relative_path is the current dir
-            if directory in [".", "./"] and relative_path not in [".", "./"]:
-                if not any(
-                    (
-                        fnmatch(relative_path, path_pattern)
-                        for path_pattern in (
-                            DATA_MODEL_PATH_PATTERN,
-                            HELPERS_PATH_PATTERN,
-                            LUTS_PATH_PATTERN,
-                            RULES_PATH_PATTERN,
-                            PACKS_PATH_PATTERN,
-                            POLICIES_PATH_PATTERN,
-                            QUERIES_PATH_PATTERN,
-                        )
-                    )
-                ):
-                    logging.debug("Skipping path %s", relative_path)
-                    continue
-            for filename in sorted(file_list):
-                # Skip hidden files
-                if filename.startswith("."):
-                    continue
-                spec_filename = os.path.abspath(os.path.join(relative_path, filename))
-                # skip loading files that have already been imported
-                if spec_filename in loaded_specs:
-                    continue
-                # Dont load files that are explictly ignored
-                relative_name = os.path.normpath(os.path.join(relative_path, filename))
-                if relative_name in ignored_normalized:
-                    logging.info("ignoring file %s", relative_name)
-                    continue
-                loaded_specs.append(spec_filename)
-                if fnmatch(filename, "*.y*ml"):
-                    with open(spec_filename, "r") as spec_file_obj:
-                        try:
-                            yield spec_filename, relative_path, yaml.load(spec_file_obj), None
-                        except (YAMLParser.ParserError, YAMLScanner.ScannerError) as err:
-                            # recreate the yaml object and yield the error
-                            yaml = YAML(typ="safe")
-                            yield spec_filename, relative_path, None, err
-                if fnmatch(filename, "*.json"):
-                    with open(spec_filename, "r") as spec_file_obj:
-                        try:
-                            yield spec_filename, relative_path, json.load(spec_file_obj), None
-                        except ValueError as err:
-                            yield spec_filename, relative_path, None, err
-
-
 def datetime_converted(obj: Any) -> Any:
     """A helper function for dumping spec files to JSON.
 
@@ -316,6 +191,47 @@ def _convert_keys_to_lowercase(mapping: Dict[str, Any]) -> Dict[str, Any]:
     return {k.lower(): v for k, v in mapping.items()}
 
 
+def zip_analysis_chunks(args: argparse.Namespace) -> List[str]:
+    logging.info("Zipping analysis items in %s to %s", args.path, args.out)
+
+    current_time = datetime.now().isoformat(timespec="seconds").replace(":", "-")
+    zip_chunks = [
+        # note: all the files we care about have an AnalysisType field in their yml
+        # so we can ignore file patterns and leave them empty
+        ZipChunk(patterns=[], types=(DATAMODEL, RULE, POLICY, PACK, GLOBAL)),  # type: ignore
+        ZipChunk(patterns=[], types=(QUERY, SCHEDULED_RULE)),  # type: ignore
+        ZipChunk(patterns=[], types=LOOKUP_TABLE),  # type: ignore
+    ]
+
+    filenames = []
+    chunks = analysis_chunks(ZipArgs.from_args(args), zip_chunks)
+    if len(chunks) != len(zip_chunks):
+        logging.error("something went wrong")
+        return []
+    for idx, chunk in enumerate(chunks):
+        filename = f"panther-analysis-{current_time}-batch-{idx+1}.zip".format()
+        filename = add_path_to_filename(args.out, filename)
+        filenames.append(filename)
+        with zipfile.ZipFile(filename, "w", zipfile.ZIP_DEFLATED) as zip_out:
+            for name in chunk.files:
+                zip_out.write(name)
+
+    return filenames
+
+
+def add_path_to_filename(output_path: str, filename: str) -> str:
+    if output_path:
+        if not os.path.isdir(output_path):
+            logging.info(
+                "Creating directory: %s",
+                output_path,
+            )
+            os.makedirs(output_path)
+        filename = f"{output_path.rstrip('/')}/{filename}"
+
+    return filename
+
+
 def zip_analysis(args: argparse.Namespace) -> Tuple[int, str]:
     """Tests, validates, and then archives all policies and rules into a local zip file.
 
@@ -337,33 +253,17 @@ def zip_analysis(args: argparse.Namespace) -> Tuple[int, str]:
     # The colon character is not valid in filenames.
     current_time = datetime.now().isoformat(timespec="seconds").replace(":", "-")
     filename = "panther-analysis-{}.zip".format(current_time)
-    if args.out:
-        if not os.path.isdir(args.out):
-            logging.info(
-                "Creating directory: %s",
-                args.out,
-            )
-            os.makedirs(args.out)
-        filename = args.out.rstrip("/") + "/" + filename
+    filename = add_path_to_filename(args.out, filename)
+
+    typed_args = ZipArgs.from_args(args)
+    chunks = analysis_chunks(typed_args)
+    if len(chunks) != 1:
+        logging.error("something went wrong")
+        return 1, ""
     with zipfile.ZipFile(filename, "w", zipfile.ZIP_DEFLATED) as zip_out:
-        # Always zip the helpers and data models
-        analysis = []
-        files: Set[str] = set()
-        for (file_name, f_path, spec, _) in list(
-            load_analysis_specs(
-                [args.path, HELPERS_LOCATION, DATA_MODEL_LOCATION], args.ignore_files
-            )
-        ):
-            if file_name not in files:
-                analysis.append((file_name, f_path, spec))
-                files.add(file_name)
-                files.add("./" + file_name)
-        analysis = filter_analysis(analysis, args.filter, args.filter_inverted)
-        for analysis_spec_filename, dir_name, analysis_spec in analysis:
-            zip_out.write(analysis_spec_filename)
-            # datamodels may not have python body
-            if "Filename" in analysis_spec:
-                zip_out.write(os.path.join(dir_name, analysis_spec["Filename"]))
+        for name in chunks[0].files:
+            zip_out.write(name)
+
     return 0, filename
 
 
@@ -380,11 +280,29 @@ def upload_analysis(backend: BackendClient, args: argparse.Namespace) -> Tuple[i
         A tuple of return code and the archive filename.
     """
 
+    if args.batch:
+        if not args.skip_tests:
+            return_code, _ = test_analysis(args)
+            if return_code != 0:
+                return return_code, ""
+
+        for idx, archive in enumerate(zip_analysis_chunks(args)):
+            batch_idx = idx + 1
+            logging.info("Uploading Batch %d...", batch_idx)
+            upload_zip(backend, args, archive)
+            logging.info("Uploaded Batch %d", batch_idx)
+
+        return 0, ""
+
     return_code, archive = zip_analysis(args)
     if return_code != 0:
         return return_code, ""
-    return_archive_fname = ""
 
+    return upload_zip(backend, args, archive)
+
+
+def upload_zip(backend: BackendClient, args: argparse.Namespace, archive: str) -> Tuple[int, str]:
+    return_archive_fname = ""
     # extract max retries we should handle
     max_retries = 10
     if args.max_retries > 10:
@@ -1073,42 +991,6 @@ def print_summary(
             print(err_message.format(spec_filename, spec_error))
 
 
-def filter_analysis(
-    analysis: List[Any], filters: Dict[str, List], filters_inverted: Dict[str, List]
-) -> List[Any]:
-    if filters is None:
-        return analysis
-
-    filtered_analysis = []
-    for file_name, dir_name, analysis_spec in analysis:
-        if fnmatch(dir_name, HELPERS_PATH_PATTERN):
-            logging.debug("auto-adding helpers file %s", os.path.join(file_name))
-            filtered_analysis.append((file_name, dir_name, analysis_spec))
-            continue
-        if fnmatch(dir_name, DATA_MODEL_PATH_PATTERN):
-            logging.debug("auto-adding data model file %s", os.path.join(file_name))
-            filtered_analysis.append((file_name, dir_name, analysis_spec))
-            continue
-        match = True
-        for key, values in filters.items():
-            spec_value = analysis_spec.get(key, "")
-            spec_value = spec_value if isinstance(spec_value, list) else [spec_value]
-            if not set(spec_value).intersection(values):
-                match = False
-                break
-        for key, values in filters_inverted.items():
-            spec_value = analysis_spec.get(key, "")
-            spec_value = spec_value if isinstance(spec_value, list) else [spec_value]
-            if set(spec_value).intersection(values):
-                match = False
-                break
-
-        if match:
-            filtered_analysis.append((file_name, dir_name, analysis_spec))
-
-    return filtered_analysis
-
-
 # pylint: disable=too-many-locals,too-many-statements
 def classify_analysis(
     specs: List[Tuple[str, str, Any, Any]]
@@ -1396,6 +1278,13 @@ def _print_test_result(
 def setup_parser() -> argparse.ArgumentParser:
     # pylint: disable=too-many-statements,too-many-locals
     # setup dictionary of named args for some common arguments across commands
+    batch_uploads_name = "--batch"
+    batch_uploads_arg: Dict[str, Any] = {
+        "action": "store_true",
+        "default": False,
+        "required": False,
+        "help": "When set your upload will be broken down into multiple zip files",
+    }
     filter_name = "--filter"
     filter_arg: Dict[str, Any] = {"required": False, "metavar": "KEY=VALUE", "nargs": "+"}
     kms_key_name = "--kms-key"
@@ -1593,6 +1482,7 @@ def setup_parser() -> argparse.ArgumentParser:
     upload_parser.add_argument(ignore_extra_keys_name, **ignore_extra_keys_arg)
     upload_parser.add_argument(ignore_files_name, **ignore_files_arg)
     upload_parser.add_argument(available_destination_name, **available_destination_arg)
+    upload_parser.add_argument(batch_uploads_name, **batch_uploads_arg)
     upload_parser.set_defaults(func=func_with_backend(upload_analysis))
 
     # -- delete command
